/*
 * @(#)IdentityGenerator.java	2011-9-24
 *
 * Copyright (c) 2011. All Rights Reserved.
 *
 */

package org.javaclub.jorm.id;

import java.io.Serializable;
import java.lang.reflect.Field;

import org.javaclub.jorm.Session;
import org.javaclub.jorm.jdbc.sql.Dialect;
import org.javaclub.jorm.jdbc.sql.impl.MySQLDialect;
import org.javaclub.jorm.jdbc.sql.impl.PostgreSQLDialect;
import org.javaclub.jorm.jdbc.sql.impl.SQLServerDialect;

/**
 * IdentityGenerator for the stratyge 'identity', this stratyge is for auto-generated keys such as 
 * auto_increment(MySQL)、identity(MSSQL) etc.
 *
 * @author <a href="mailto:gerald.chen.hz@gmail.com">Gerald Chen</a>
 * @version $Id: IdentityGenerator.java 488 2011-09-27 09:28:15Z gerald.chen.hz $
 */
public class IdentityGenerator implements IdentifierGenerator {
	
	private Dialect dialect;

	public synchronized Serializable generate(Session session, Object object,
			Field field) {
		Object nextId = null;
		if ((dialect instanceof MySQLDialect) 
			|| (dialect instanceof SQLServerDialect) 
			|| (dialect instanceof PostgreSQLDialect)) {
			nextId = session.unique(
					getDialect().nextIdentitySql(session.getDefaultDatabase(),
												 object.getClass()));
		}
		
		return IdentifierGeneratorFactory.getGeneratedIdentity(nextId, field.getType());
	}

	public Dialect getDialect() {
		return this.dialect;
	}

	public void setDialect(Dialect dialect) {
		this.dialect = dialect;

	}

}
